#encoding: UTF-8 
=begin
/***************************************************************************
 *   ©2016-2017 Michael Uplawski <michael.uplawski@uplawski.eu>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
=end
require 'yaml'
require 'singleton'
require 'ostruct'
require_relative 'file_checking'
require_relative 'logging'
require_relative 'translating'

class Configuration
  include File_Checking
  include Singleton
  include Translating
  
  # class-level logger is not more useful in the singleton,
  # than the object-level logger, but what the heg.
  
  self.extend(Logging)
  @@log = init_logger()

  # default configuration file
  @@config_file = File::dirname(__FILE__) << File::Separator << 'config'

  # do initializations
  def initialize()
    # Use class-level logger for now.
    # init_logger()
    @log = @@log
  end

  # Configure with the command-line arguments.
  def set(options)
    @log.debug('options are: ' << options.to_s )
    # User-provided configuration-file?
    if(options['config'])
      cf = options['config'] 
      msg = check_file(cf, :file, :readable)
      if(!msg) 
        @@config_file = cf
      else
        msg = trl("The file %s " << msg.split[1,100].join(' ')) %msg.split[0]
        @log.error(trl("ERROR! Unsuitable file") << ' ' << msg)
        give_up
      end
    end

    @log.debug('config-file is ' << @@config_file)
    # read defaults from configuration-file
    co = OpenStruct.new(YAML::load_file(@@config_file))
    # merge and overwrite with the command-line arguments
    @config = co.to_h.merge(options.to_h)
    @log.debug('config is now: ' << @config.to_s )
    verify
  end

  # return any value stored in @config
  def method_missing(msg, *args)
    ms = msg.to_sym
    # Exception-handling is not a control-structure.
    # This is.
    if @config[ms]
      return @config[ms]
    else
      return nil
    end
  end

  private

  # exit on error
  def give_up
    @log.error("\t" << trl("Aborting. Bye!"))
    exit false
  end

  # Do a maximum of verifications. 
  # Not everything is possible before the database has been read.
  def verify
    msg = nil
    # ensure :source is set
    if(! @config[:source] )
      msg = trl('missing argument %s') %'source' 
      # either :name or :list are needed in addition
    elsif (!@config[:name] && !@config[:list])
      msg = trl("Either --name (-n) or --list (-l) is needed as additional program-argument" )
    end

    # avoid contradicting options
    if !msg && @config[:target] && @config[:out]
      msg = trl("contradictory arguments (%s):" %['target', 'out'])
    end

    if msg
      @log.error msg
      @log.error(trl("Start this program with parameter -h or --help to see the usage-message.") )
      give_up
    end
  end

end

#eof
