#encoding: UTF-8 
=begin
/***************************************************************************
 *   ©2016-2017 Michael Uplawski <michael.uplawski@uplawski.eu>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
=end

# This is not a module.
# TODO get rid of the ambivalence.
# This class represents the mapping between SQLite data-types and those in
# dBase. It is basically a decorator for an Array of OpenStruct objects.
# Mapping-objects have access to the configuration.
# TODO: Decide if this needs to be a singleton (one table only) or if it might,
# in the future, make sense to preview several mapping-objects for the
# transformation of several tables in a row.
class Mapping
  # Depending on the decisions for the future uses of this class,
  # it might or might not be better to move the logger to class-level.
  include Logging

  @@TMap = {
    'INT' => 1, 
    'TINYINT' => 1, 
    'SMALLINT' => 1, 
    'MEDIUMINT' => 1, 
    'BIGINT' => 1, 
    'UNSIGNED BIGINT' => 1, 
    'VARCHAR' => 0, 
    'CHARACTER' => 0, 
    'VARYING CHARACTER' => 0, 
    'LONGVARCHAR' => 0, 
    'TEXT' => 0, 
    'BLOB' => 0, 
    'INTEGER' => 1, 
    'FLOAT' => 2, 
    'REAL' => 2, 
    'DOUBLE' => 2
  } 

  # Constructor
  # Creates a mapping for the table of name config.name and based on the
  # table_info.
  def initialize config, table_info, content
    # TODO: Veryfy the need for an object level logger. Move to class-level,
    # when more than one object of this class can be created.
    init_logger()

    @table_name = config.name
    @mapping = []
    @content = content
    @date_fields = config.date ? config.date : [] 
    @time_fields = config.time ? config.time : []

    table_info.each_with_index do |field, i|
      @log.debug('field is ' << (field ? field.to_s : ' N I L ') )
      field_name = field['name']

      ftype = nil 

      # Enforce Text field for dates and times.
      ftype = 0 if  @date_fields.include?(field_name)
      ftype = 0 if @time_fields.include?(field_name) 

      # Do as you must for the remainder
      ftype ||= @@TMap.detect{|key, value| field['type'].upcase.start_with?(key) }[1]

      @mapping << OpenStruct.new(:name => field_name, :type => ftype, :index => i)
    end
  end
  def [](*args)
    @mapping.detect do |field|
      field.index == args[0]
    end
  end

  def each(&b)
    @mapping.each do |field|
      yield(field)
    end
  end

end
