#encoding: UTF-8 
=begin
/***************************************************************************
 *   ©2016-2017 Michael Uplawski <michael.uplawski@uplawski.eu>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
=end


require_relative 'file_checking'
require_relative 'logging'
require_relative 'configuration'
require_relative 'mapping'
require 'shp'
require 'sqlite3'
require_relative 'argparser'

=begin
        The main program class. Does it.
=end

class SQLite2DBF
 
  include Logging
  include Translating

  def initialize(*args)
    options = ArgParser::parse(args)
    @config = Configuration.instance()
    @config.set(options)
    # Object level logger. “There can only be one!”
    init_logger
    level = (@config.debug ? Logger::DEBUG : @log.level)
    @log.level = level	

    @date_fields = @config.date ? @config.date : []
    @time_fields = @config.time ? @config.time : []

    if(@config.source)
      @dbf_path = nil
      sqlite_file = @config.source	

      msg = File_Checking::file_check(sqlite_file, :exist, :readable)
      if msg
        @log.error(trl("ERROR! Cannot read the source-file" ) << ": " << msg)
        exit false
      end

      SQLite3::Database.new(sqlite_file) do |db|
        tables = list(db)
        if(@config.list)
          puts "\n" << trl("Tables in the database") << ":\n\t"  << tables.join("\n\t") << "\n\n"
          exit true
        elsif tables.include?(@config.name)
          @mapping = mapping(db)
        else
          @log.error(trl("Verify table-name! %s is not found in the database!") %(@config.name))
          @log.error(trl("Tables are %s") %tables.join(', ') )

          exit false
        end

        # dbf_file is eather named explicitly
        dbf_file = @config.target if @config.target
        # .., derived from the table-name
        dbf_file = @config.out << File::Separator << @config.name if(@config.out)
        # .., or identical with the original sqlite-file, minus extension, plus dbf
        dbf_file ||= File.dirname(sqlite_file) << File::Separator << File.basename(sqlite_file, '.*') << '.dbf'

        msg = nil

        # check if dbf_file can be used
        if(File.exist?(dbf_file))
          msg = File_Checking.file_check(dbf_file, :file, :writable)
        elsif(File.exist?(File.dirname(dbf_file))) 
          msg = File_Checking.file_check(File.dirname(dbf_file), :directory, :writable)
        end

        # then do it
        if(!msg)
          @log.debug('will transform ' << sqlite_file << ' to  ' << dbf_file.to_s)
          transform(db, dbf_file)
        else
          # or not
          msg = trl("ERROR! Unsuitable file") << " : %s" %msg
          @log.error(msg)
          exit false
        end
      end
    else
      log.error trl("ERROR! Source-file is a mandatory program parameter!")
      log.error trl("Start this program with parameter -h or --help to see the usage-message.")  
    end

  end

  private

  # Create a list of table-names from SQLite
  def list(db)
    table_names = []
    table_names = db.execute('select name from sqlite_master WHERE type="table"').map {|tn| tn[0]} 
    @log.debug('sqlite contains ' << table_names.size.to_s << ' table(s): ' << table_names.join(', '))
    return table_names
  end

  def mapping(db)
    return Mapping.new(@config, db.table_info(@config.name), db.execute('select * from ' << @config.name ) )
  end

  def create_base(dbase)
    @log.debug('creating fields')
    @mapping.each do |field|
      dbase.add_field(field.name, field.type, 200, 0)
    end
  end

  # create a date-value from value
  def date(db, value)
    dbf_value = db.execute("select strftime('%Y-%m-%d %H:%M:%S', " << value.to_s << ", 'unixepoch')").join
    # ... or not.
    @log.warn(trl("ATTN! SQLite2DBF cannot convert the date-value %s into DBF-compatible format.") %value.to_s) if dbf_value.start_with?('-')
    dbf_value = db.execute("select strftime('%s', " << value.to_s << ")").join if dbf_value.start_with?('-')
  end

  # create a time-value from value
  def time(db, value)
    db.execute("select strftime('%s', " << value.to_s << ", 'unixepoch')").join
  end

  def transform(db, dbf_file)
    begin
      tname = @config.name
      table_info = db.table_info(tname) 
      content = db.execute('select * from ' << tname ) 
      @log.debug('have content')

      if(:table == dbf_file)	
        dbf_file = @dbf_path << File::Separator << tname << (content.empty? ? '_empty' : '')<< '.dbf'
      end
      @log.debug('dbf will be ' << dbf_file)
      dbase = SHP::DBF.create(dbf_file) 

      create_base(dbase)

      content.each_with_index do |row, record_no|
        @log.debug('row is ' << row.to_s)

        row.each_with_index do |svalue, field_index|
          @log.debug('object is ' << @mapping[field_index].to_s)
          dbf_type = @mapping[field_index].type
          field_name = @mapping[field_index].name

          if(svalue && !svalue.to_s.empty? && dbf_type)
            svalue = date(db, svalue) if @date_fields.include?(field_name)
            svalue = time(db, svalue) if @time_fields.include?(field_name)

            @log.debug('field is ' << field_name << ', svalue is ' << svalue.to_s << ', type is ' << dbf_type.to_s)
            # This is Ruby. But if you do not like the verbosity 
            # remove the magic numbers and call them text, int and float or
            # similar.
            begin
              case dbf_type
              when 0
                dbase.write_string_attribute(record_no, field_index, svalue ) 
              when 1
                dbase.write_integer_attribute(record_no, field_index, svalue) 
              when 2
                dbase.write_double_attribute(record_no, field_index, svalue) 
              else
                dbase.write_null_attribute(record_no, field_index)
              end
            rescue StandardError => ex
              @log.warn(trl("ATTN! Field %s - Cannot write value of type %s") %[field_name, dbf_type] << ': ' << ex.message )
            end
          end
        end
      end
      puts trl('DONE. Bye.')
    rescue SQLite3::Exception => er
      @log.error trl("ERROR! Cannot read the source-file") << ' (' << db.filename << "): %s" %er.message << '.'
    end
  end
end
